/*
 * ClipboardDocFrame.java
 *
 * Copyright 2003 Steve Held
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */

package makedocjr.clipboard;

import javax.swing.*;
import java.awt.*;
import java.awt.event.*;
import java.awt.datatransfer.*;
import javax.swing.border.*;
import java.io.*;
import makedocjr.*;

/**
 * Title:        ClipboardDocFrame
 * Description:  Frame for ClipboardDoc
 * Copyright:    Copyright (c) 2003
 * Company:
 * @author Steve Held
 * @version 1.0
 */


public class ClipboardDocFrame extends JFrame {

	private static String lineSeparator = System.getProperty("line.separator");
	private static String fileSeparator = System.getProperty("file.separator");
	private static String[] categories = {"0", "1", "2", "3", "4", "5", "6", "7",
							  "8", "9", "10", "11", "12", "13", "14", "15"};
	private static byte MAX_TITLE_LENGTH = 31; // longer ones get cut off
	private static byte MAX_FILENAME_ATTEMPTS = 99; // if need to try a new name
			// also sets the maximum number of files with the same name

	private JPanel jPanel1 = new JPanel();
	private JLabel jLabelCat = new JLabel();
	private JButton jButtonCreate = new JButton();
	private GridBagLayout gridBagLayout1 = new GridBagLayout();
	private JButton jButtonExit = new JButton();
	private JLabel jLabelStatus = new JLabel();
	private JComboBox jComboBoxCategory = new JComboBox(categories);
	private JCheckBox jCheckBoxCompressed = new JCheckBox();
	private JCheckBox jCheckBoxPrivate = new JCheckBox();
	private JLabel jLabelDirectory = new JLabel();
	private JTextField jTextFieldDirectory = new JTextField();
	private JButton jButtonDirectory = new JButton();
	private Component component1;
	private Component component2;
	private Border border1;
	private String status = "ClipboardDoc v1.0.1";
	private JFileChooser jFileChooser;
	private MakeDocJr makeDocJr = new MakeDocJr();
	private EncodeOptions options;
	private String optionsName = "options.ser";

	public ClipboardDocFrame() {
		// read serialized EncodeOptions or create new EncodeOptions
		FileInputStream fis = null;
		ObjectInputStream ois = null;
		try {
			fis = new FileInputStream( optionsName );
			ois = new ObjectInputStream(fis);
			options = (EncodeOptions) ois.readObject();
		} catch (Exception e) {
			options = new EncodeOptions();
		} finally {
			try {
				if (ois != null)
					ois.close();
			} catch (Exception e) {}
			try {
				if (fis != null)
					fis.close();
			} catch (Exception e) {}
		}
		makeDocJr.setEncodeOptions(options);
		try {
			jbInit();
		}
		catch(Exception e) {
			e.printStackTrace();
		}
	}


	private void jbInit() throws Exception {
		border1 = BorderFactory.createCompoundBorder(BorderFactory.createEtchedBorder(Color.white,new Color(134, 134, 134)),BorderFactory.createEmptyBorder(7,7,7,7));
		this.setTitle("ClipboardDoc");
		this.setDefaultCloseOperation(WindowConstants.DO_NOTHING_ON_CLOSE);
		component1 = Box.createVerticalStrut(8);
		component2 = Box.createVerticalStrut(8);
		jLabelCat.setText("Category:");
		jComboBoxCategory.setSelectedIndex(options.getCategory());
		jButtonCreate.setText("Create");
		jButtonCreate.addActionListener(new java.awt.event.ActionListener() {
			public void actionPerformed(ActionEvent e) {
				jButtonCreate_actionPerformed(e);
			}
		});
		jPanel1.setLayout(gridBagLayout1);
		jButtonExit.setText("Exit");
		jButtonExit.addActionListener(new java.awt.event.ActionListener() {
			public void actionPerformed(ActionEvent e) {
				jButtonExit_actionPerformed(e);
			}
		});
		jLabelStatus.setBackground(Color.white);
		jLabelStatus.setBorder(BorderFactory.createEtchedBorder());
		jLabelStatus.setText(status);
		jCheckBoxCompressed.setSelected(options.getCompress());
		jCheckBoxCompressed.setText("Compressed");
		jCheckBoxPrivate.setText("Private");
		jCheckBoxPrivate.setSelected(options.getMakePrivate());
		jLabelDirectory.setText("Save in:");
		jTextFieldDirectory.setMinimumSize(new Dimension(220, 21));
		jTextFieldDirectory.setPreferredSize(new Dimension(220, 21));
		jTextFieldDirectory.setText(options.getDirectoryPath());
		jButtonDirectory.setMaximumSize(new Dimension(30, 20));
		jButtonDirectory.setMinimumSize(new Dimension(30, 20));
		jButtonDirectory.setPreferredSize(new Dimension(30, 20));
		jButtonDirectory.setMnemonic('0');
		jButtonDirectory.setText("...");
		jButtonDirectory.addActionListener(new java.awt.event.ActionListener() {
			public void actionPerformed(ActionEvent e) {
				jButtonDirectory_actionPerformed(e);
			}
		});
		jPanel1.setBorder(border1);
		this.getContentPane().add(jPanel1, BorderLayout.CENTER);
		jPanel1.add(jButtonCreate,                                                           new GridBagConstraints(0, 6, 1, 1, 0.0, 0.0
			,GridBagConstraints.WEST, GridBagConstraints.NONE, new Insets(0, 0, 0, 0), 0, 0));
		jPanel1.add(jLabelCat,                                                          new GridBagConstraints(0, 2, 1, 2, 0.0, 0.0
			,GridBagConstraints.WEST, GridBagConstraints.NONE, new Insets(0, 0, 0, 0), 0, 0));
		jPanel1.add(jLabelDirectory,                                new GridBagConstraints(0, 0, 1, 1, 0.0, 0.0
			,GridBagConstraints.WEST, GridBagConstraints.NONE, new Insets(0, 0, 0, 0), 0, 0));
		jPanel1.add(jTextFieldDirectory,                                    new GridBagConstraints(1, 0, 2, 1, 0.0, 0.0
			,GridBagConstraints.WEST, GridBagConstraints.HORIZONTAL, new Insets(0, 0, 0, 0), 0, 0));
		jPanel1.add(jLabelStatus,                                  new GridBagConstraints(0, 8, 5, 1, 0.0, 0.0
			,GridBagConstraints.WEST, GridBagConstraints.HORIZONTAL, new Insets(0, 0, 0, 0), 0, 0));
		jPanel1.add(jButtonDirectory,                     new GridBagConstraints(3, 0, 1, 1, 0.0, 0.0
			,GridBagConstraints.WEST, GridBagConstraints.NONE, new Insets(0, 10, 0, 0), 0, 0));
		jPanel1.add(jButtonExit,                  new GridBagConstraints(1, 6, 3, 1, 0.0, 0.0
			,GridBagConstraints.WEST, GridBagConstraints.NONE, new Insets(0, 20, 0, 0), 0, 0));
		jPanel1.add(jComboBoxCategory,             new GridBagConstraints(1, 2, 1, 1, 0.0, 0.0
			,GridBagConstraints.WEST, GridBagConstraints.NONE, new Insets(0, 0, 0, 0), 0, 0));
		jPanel1.add(jCheckBoxCompressed,           new GridBagConstraints(0, 4, 3, 1, 0.0, 0.0
			,GridBagConstraints.WEST, GridBagConstraints.NONE, new Insets(0, 0, 0, 0), 0, 0));
		jPanel1.add(jCheckBoxPrivate,       new GridBagConstraints(0, 5, 1, 1, 0.0, 0.0
			,GridBagConstraints.WEST, GridBagConstraints.NONE, new Insets(0, 0, 0, 0), 0, 0));
		jPanel1.add(component1,     new GridBagConstraints(1, 1, 2, 1, 0.0, 0.0
			,GridBagConstraints.CENTER, GridBagConstraints.NONE, new Insets(0, 0, 0, 0), 0, 0));
		jPanel1.add(component2,    new GridBagConstraints(0, 7, 1, 1, 0.0, 0.0
			,GridBagConstraints.CENTER, GridBagConstraints.NONE, new Insets(0, 0, 0, 0), 0, 0));
	}


	/**Overridden so we can exit when window is closed*/
	protected void processWindowEvent(WindowEvent e) {
		super.processWindowEvent(e);
		if (e.getID() == WindowEvent.WINDOW_CLOSING) {
			exit();
		}
	}


	private void setOptions() {
		options.setCategory( (byte) jComboBoxCategory.getSelectedIndex() );
		options.setMakePrivate( jCheckBoxPrivate.isSelected() );
		options.setCompress( jCheckBoxCompressed.isSelected() );
		options.setDirectoryPath( jTextFieldDirectory.getText().trim() );
	}


	// serialize the EncodeOptions
	private void saveOptions() {
		FileOutputStream fos = null;
		ObjectOutputStream oos = null;
		try {
			fos = new FileOutputStream(optionsName);
			oos = new ObjectOutputStream(fos);
			oos.writeObject(options);
			oos.flush();
		} catch (Exception e) { // FileNotFoundException, IOException
			// options won't be saved
		} finally {
			try {
				if (oos != null)
					oos.close();
				if (fos != null)
					fos.close();
			} catch (IOException ioe) {}
		}
	}


	private void exit() {
		setOptions();
		saveOptions();
		System.exit(0);
	}


	void jButtonDirectory_actionPerformed(ActionEvent e) {
		if ( jFileChooser == null ) {
			jFileChooser = new JFileChooser(jTextFieldDirectory.getText());
			jFileChooser.setFileSelectionMode(JFileChooser.DIRECTORIES_ONLY);
		}
		if (JFileChooser.APPROVE_OPTION == jFileChooser.showSaveDialog(this)) {
			String dir = jFileChooser.getSelectedFile().getPath();
			jTextFieldDirectory.setText(dir);
		}

	}


	void jButtonExit_actionPerformed(ActionEvent e) {
		exit();
	}


	void jButtonCreate_actionPerformed(ActionEvent e) {
		boolean clipboardNotEmpty = true;
		Clipboard clipboard = Toolkit.getDefaultToolkit().getSystemClipboard();
		String text = null;
		try {
			text = (String) clipboard.getContents(this).getTransferData(
													DataFlavor.stringFlavor);
		} catch (Exception ex) {
			// UnsupportedFlavorException
			// IOException
			// NullPointerException (thrown before above exceptions?)
			// all handled below (text == null)
		}
		if ( text == null ) {
			clipboardNotEmpty = false;
			status = "The clipboard contains no text."; // might have other data
		}
		String directory = jTextFieldDirectory.getText().trim();
		if (clipboardNotEmpty) {
			boolean directoryExists = true;
			// verify directory exists; if doesn't exist, try to create it
			File dir = new File(directory);
			if ( !dir.isDirectory() ) {
				directoryExists = dir.mkdirs();
			}
			if (directoryExists) {
				try {
					// create a unique filename (and title)
					int textLength = text.length();
					byte titleLength = 0;

					if (textLength < MAX_TITLE_LENGTH)
						titleLength = (byte) textLength;
					else
						titleLength = MAX_TITLE_LENGTH;

					String title = text.substring(0, titleLength);
					title = removeLineBreaks(title);
					// create a unique filename by prefixing an int if necessary
					String filename = makeValidFilename(title);
					File file = new File(directory, filename + ".pdb");
					int renameTry = 1;
					while ( !file.createNewFile() &&
							(renameTry <= MAX_FILENAME_ATTEMPTS) ) {
						// numbers are placed first as easy fix to getting them
						// cut-off if title.length = 31
						StringBuffer filenameBuffer = new StringBuffer(Integer.toString(renameTry));
						filenameBuffer.append(filename).append(".pdb");
						file = new File(directory, filenameBuffer.toString());
						++renameTry;
					}
					if (file.exists()) {
						// encode it

						// update status
						SwingUtilities.invokeLater(new Runnable(){
							public void run(){
								jLabelStatus.setText("Creating...");
							}
						});

						// if file was not unique, title probably won't be either
						if (renameTry > 1) {
							title = String.valueOf(renameTry -1) + title;
						}

						makeDocJr.setTitle(title);
						makeDocJr.setText(text);
						setOptions();

						// create thread to do work while UI is updated
						EncodeThread encodeThread = new EncodeThread(file);
						encodeThread.start();
					} else {
						status = "Could not create the file " + filename + ".pdb";
					}
				} catch (IOException ioe) {
					status = "I/O error creating the file.";
				}
			} else {
				status = "Could not create your directory.";
			}
		}
		jLabelStatus.setText(status);
		repaint();
	}


	// Creates a filename consisting of only letters, digits and underscores
	// that will be valid on any operating system
	private String makeValidFilename(String filename) {
		char[] chars = filename.toCharArray();
		for (int i = 0; i < chars.length; i++) {
			if ( !Character.isLetterOrDigit(chars[i]) ) {
				chars[i] = '_';
			}
		}
		return new String(chars);
	}


	// Removes \n from title
	private String removeLineBreaks(String title) {
		char[] chars = title.toCharArray();
		for (int i = 0; i < chars.length; i++) {
			if ( chars[i] == '\n' ) {
				chars[i] = ' ';
			}
		}
		return new String(chars);
	}


	// This thread allows status to be updated on UI
	private class EncodeThread extends Thread {
		File file;

		EncodeThread(File file) {
			this.file = file;
		}

		public void run() {
			try {
				makeDocJr.encode(file);
				status = "Created " + file.getName();
			} catch (IOException ioe) {
				status = "Could not create the file " + file.getName();
			}
			SwingUtilities.invokeLater(new Runnable(){
				public void run(){
					jLabelStatus.setText(status);
				}
			});
		}
	}

}